import zmq


def zmq_export(sock: zmq.Socket, topic: str, data, datatype: str = "pyobj"):
    """
    Author:
    Alexander Heilmeier & Tim Stahl

    Description:
    Sends data via ZMQ.

    Inputs:
    sock:       ZMQ socket (see below how to create it)
    topic:      ZMQ topic to use
    data:       data to send
    datatype:   string that indicates if it should be sent as Python object (pyobj), json (json) or string (str)

    Hint: To send an object as string it must be converted to a string at first. Conversion of Python objects to json
    objects is handled by PyZMQ and therefore must not be done by hand if sending a json.

    How to create a ZMQ socket to export data?
    import zmq
    zmq_context = zmq.Context()
    sock = zmq_context.socket(zmq.PUB)
    sock.bind("tcp://*:%s" % port)
    """

    # ------------------------------------------------------------------------------------------------------------------
    # FUNCTION BODY ----------------------------------------------------------------------------------------------------
    # ------------------------------------------------------------------------------------------------------------------

    sock.send_string(topic, zmq.SNDMORE)

    if datatype == "pyobj":
        sock.send_pyobj(data)
    elif datatype == "json":
        sock.send_json(data)
    elif datatype == "str":
        sock.send_string(data)
    else:
        raise ValueError("Specified datatype is not supported!")


# testing --------------------------------------------------------------------------------------------------------------
if __name__ == "__main__":
    pass



import socket
from typing import Union


def udp_import(sock: socket.socket, data_size: int, use_buffer: bool = False) -> Union[bytes, list, None]:
    """
    Author:
    Alexander Heilmeier

    Description:
    Import data via UDP.

    Inputs:
    sock:           UDP socket (see below how to create it)
    data_size:      data size per message in byte (not bit!)
    use_buffer:     flag to show if the buffer should be returned completely (True) or only the newest message (False)

    How to create a UDP socket to import data?
    import socket
    sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
    sock.bind((own_external_ip, listening_port))
    sock.setblocking(False)
    """

    # ------------------------------------------------------------------------------------------------------------------
    # FUNCTION BODY ----------------------------------------------------------------------------------------------------
    # ------------------------------------------------------------------------------------------------------------------

    data_b = None
    buffer = []

    try:
        while True:
            data_b, addr = sock.recvfrom(data_size)

            if use_buffer:
                buffer.append(data_b)

    except socket.error:
        pass

    if use_buffer and buffer:
        return buffer
    elif use_buffer:
        return None
    else:
        return data_b


# testing --------------------------------------------------------------------------------------------------------------
if __name__ == "__main__":
    pass



import zmq


def zmq_import(sock: zmq.Socket, blocking: bool = False, datatype: str = "pyobj", use_buffer: bool = False):
    """
    Author:
    Alexander Heilmeier & Tim Stahl

    Description:
    Handles incoming ZMQ messages.

    Inputs:
    sock:       ZMQ socket (see below how to create it)
    blocking:   set if socket should be blocking (i.e. wait until new message is received) or not
    datatype:   string that indicates if it should be received as Python object (pyobj), json (json) or string (str)
    use_buffer: flag to indicate if the buffer should be returned completely (True) or only the latest message (False)

    Hint: Conversion of json objects to their original data type is handled by PyZMQ and therefore must not be done by
    hand.

    How to create a ZMQ socket to import data?
    import zmq
    zmq_context = zmq.Context()
    sock = zmq_context.socket(zmq.PUB)
    sock.connect("tcp://%s:%s" % (ip, port))
    sock.setsockopt_string(zmq.SUBSCRIBE, zmq_topic])
    """

    # ------------------------------------------------------------------------------------------------------------------
    # Specify called receive handle ------------------------------------------------------------------------------------
    # ------------------------------------------------------------------------------------------------------------------

    if datatype == "pyobj":
        sock_recv_fct = lambda **kwargs: sock.recv_pyobj(**kwargs)
    elif datatype == "json":
        sock_recv_fct = lambda **kwargs: sock.recv_json(**kwargs)
    elif datatype == "str":
        sock_recv_fct = lambda **kwargs: sock.recv_string(**kwargs)
    else:
        raise ValueError("Specified datatype is not supported!")

    # ------------------------------------------------------------------------------------------------------------------
    # FUNCTION BODY ----------------------------------------------------------------------------------------------------
    # ------------------------------------------------------------------------------------------------------------------

    data = None
    buffer = []

    # non blocking socket to empty buffer
    try:
        while True:
            sock.recv_string(flags=zmq.NOBLOCK)
            data = sock_recv_fct(flags=zmq.NOBLOCK)

            if use_buffer:
                buffer.append(data)

    except zmq.Again:
        pass

    # if no data was received above and blocking is True, wait for new data
    if blocking and not use_buffer and data is None:
        sock.recv_string()
        data = sock_recv_fct()

    elif blocking and use_buffer and not buffer:
        sock.recv_string()
        data = sock_recv_fct()
        buffer.append(data)

    if use_buffer and buffer:
        return buffer
    elif use_buffer:
        return None
    else:
        return data


# testing --------------------------------------------------------------------------------------------------------------
if __name__ == "__main__":
    pass



import ad_interface_functions.udp_export
import ad_interface_functions.udp_import
import ad_interface_functions.zmq_export
import ad_interface_functions.zmq_import
import ad_interface_functions.zmq_import_poll



import setuptools

"""
Package on pypi.org can be updated with the following commands:
python3 setup.py sdist bdist_wheel
sudo python3 -m twine upload dist/*
"""

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name='ad-interface-functions',
    version='0.21',
    url='https://www.ftm.mw.tum.de/en/main-research/vehicle-dynamics-and-control-systems/roborace-autonomous-motorsport/',
    author="Alexander Heilmeier, Tim Stahl",
    author_email="alexander.heilmeier@tum.de, stahl@ftm.mw.tum.de",
    description="Interface functions used in the autonomous driving stack at TUM/FTM",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    install_requires=[
        'pyzmq==19.0.2'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)",
        "Operating System :: OS Independent",
    ])
